@@ -1,2 +1,2 @@
 from ._version import __version__
-from ._why import Why
+from .api import Why
